@@ -10,28 +10,28 @@ extern crate derive_builder;
 
 extern crate pathdiff;
 
+use core::sync::atomic::AtomicBool;
 use std::collections::{HashMap, HashSet};
-//use std::error::Error;
 use std::env;
 use std::fmt;
 use std::hash::{Hash, Hasher};
 use std::path::{Path, PathBuf};
+use std::thread;
 use std::{iter, iter::Filter, slice::Iter};
 
 #[macro_use]
 extern crate serde_derive;
 
-use indexmap::{IndexMap, IndexSet};
-
 use anyhow::{Context as _, Error, Result};
 use clap::{crate_version, App, AppSettings, Arg, SubCommand};
+use indexmap::{IndexMap, IndexSet};
+use signal_hook::{consts::SIGINT, iterator::Signals};
 
 mod data;
 mod download;
 mod generate;
 mod nested_env;
 mod ninja;
-
 use generate::{get_ninja_build_file, BuildInfo, GenerateMode, Selector};
 use nested_env::{Env, MergeOption};
 use ninja::NinjaCmdBuilder;
@@ -70,6 +70,8 @@ pub struct Task {
     cmd: Vec<String>,
     #[serde(default = "default_as_true")]
     build: bool,
+    #[serde(default = "default_as_false")]
+    ignore_ctrl_c: bool,
 }
 
 impl Task {
@@ -94,7 +96,16 @@ impl Task {
                 command.arg(cmd);
             }
 
+            if self.ignore_ctrl_c {
+                IGNORE_SIGINT.store(true, std::sync::atomic::Ordering::SeqCst);
+            }
+
+            // run command, wait for status
             command.status().expect("command exited with error code");
+
+            if self.ignore_ctrl_c {
+                IGNORE_SIGINT.store(false, std::sync::atomic::Ordering::SeqCst);
+            }
         }
         Ok(())
     }
@@ -105,8 +116,13 @@ impl Task {
             cmd: self
                 .cmd
                 .iter()
-                .map(|cmd| nested_env::expand(cmd, env, nested_env::IfMissing::Ignore).unwrap())
+                .map(|cmd| {
+                    nested_env::expand(cmd, env, nested_env::IfMissing::Ignore)
+                        .unwrap()
+                        .replace("$$", "$")
+                })
                 .collect(),
+            ignore_ctrl_c: self.ignore_ctrl_c,
         }
     }
 }
@@ -1088,7 +1104,22 @@ fn main() {
     };
 }
 
+static IGNORE_SIGINT: AtomicBool = AtomicBool::new(false);
+
 fn try_main() -> Result<i32> {
+    let mut signals = Signals::new(&[SIGINT])?;
+
+    thread::spawn(move || {
+        for sig in signals.forever() {
+            println!("laze: received signal {:?}", sig);
+            if sig == SIGINT {
+                if !IGNORE_SIGINT.load(std::sync::atomic::Ordering::SeqCst) {
+                    std::process::exit(130);
+                }
+            }
+        }
+    });
+
     let matches = App::new("laze in rust")
         .version(crate_version!())
         .author("Kaspar Schleiser <kaspar@schleiser.de>")
@@ -1180,6 +1211,17 @@ fn try_main() -> Result<i32> {
                         .multiple(true)
                         .require_delimiter(true)
                         .env("LAZE_DISABLE"),
+                )
+                .arg(
+                    Arg::with_name("define")
+                        .short("D")
+                        .long("define")
+                        .help("set/override variable")
+                        .required(false)
+                        .takes_value(true)
+                        .multiple(true)
+                        .number_of_values(1)
+                        .env("LAZE_DEFINE"),
                 ),
         )
         .subcommand(
@@ -1243,6 +1285,17 @@ fn try_main() -> Result<i32> {
                         .multiple(true)
                         .require_delimiter(true)
                         .env("LAZE_DISABLE"),
+                )
+                .arg(
+                    Arg::with_name("define")
+                        .short("D")
+                        .long("define")
+                        .help("set/override variable")
+                        .required(false)
+                        .takes_value(true)
+                        .multiple(true)
+                        .number_of_values(1)
+                        .env("LAZE_DEFINE"),
                 ),
         )
         .get_matches();
@@ -1284,11 +1337,24 @@ fn try_main() -> Result<i32> {
                 None => Selector::All,
             };
 
+            println!("building {} for {}", &apps, &builders);
+
             // collect CLI selected modules
             let select = build_matches.values_of_lossy("select");
             let disable = build_matches.values_of_lossy("disable");
 
-            println!("building {} for {}", &apps, &builders);
+            // collect CLI env overrides
+            let cli_env = if build_matches.occurrences_of("define") > 0 {
+                let mut env = Env::new();
+
+                for assignment in build_matches.values_of("define").unwrap() {
+                    env = nested_env::assign_from_string(env, assignment)?;
+                }
+
+                Some(env)
+            } else {
+                None
+            };
 
             let mode = match global {
                 true => GenerateMode::Global,
@@ -1304,6 +1370,7 @@ fn try_main() -> Result<i32> {
                 apps.clone(),
                 select,
                 disable,
+                cli_env.as_ref(),
             )?;
 
             // generation of ninja build file complete.
@@ -1376,6 +1443,19 @@ fn try_main() -> Result<i32> {
                 false => GenerateMode::Local(start_relpath),
             };
 
+            // collect CLI env overrides
+            let cli_env = if task_matches.occurrences_of("define") > 0 {
+                let mut env = Env::new();
+
+                for assignment in task_matches.values_of("define").unwrap() {
+                    env = nested_env::assign_from_string(env, assignment)?;
+                }
+
+                Some(env)
+            } else {
+                None
+            };
+
             println!("building {} for {}", &apps, &builders);
             // arguments parsed, launch generation of ninja file(s)
             let builds = generate::generate(
@@ -1386,6 +1466,7 @@ fn try_main() -> Result<i32> {
                 apps.clone(),
                 select,
                 disable,
+                cli_env.as_ref(),
             )?;
 
             let builds: Vec<&(String, String, BuildInfo)> = builds
